Package: actuar
Type: Package
Title: Actuarial Functions and Heavy Tailed Distributions
Version: 3.3-7
Date: 2026-03-02
Authors@R: c(person("Vincent", "Goulet", role = c("cre", "aut"),
 	            email = "vincent.goulet@act.ulaval.ca",
		    comment = c(ORCID = "0000-0002-9315-5719")),
	     person("Sébastien", "Auclair", role = "ctb"),
	     person("Jérémy", "Déraspe", role = "ctb"),
	     person("Christophe", "Dutang", role = "aut",
	            email = "dutang@ceremade.dauphine.fr",
                    comment = c(ORCID = "0000-0001-6732-1501")),
	     person("Walter", "Garcia-Fontes", role = "ctb",
	            email = "walter.garcia@upf.edu"),
	     person("Nicholas", "Langevin", role = "ctb"),
	     person("Xavier", "Milhaud", role = "ctb"),
	     person("Tommy", "Ouellet", role = "ctb"),
	     person("Alexandre", "Parent", role = "ctb"),
	     person("Mathieu", "Pigeon", role = "aut",
	            email = "pigeon.mathieu.2@uqam.ca"),
	     person("Louis-Philippe", "Pouliot", role = "ctb"),
     	     person("Jeffrey A.", "Ryan", role = "aut",
	            email = "jeff.a.ryan@gmail.com",
		    comment = "Package API"),
	     person("Robert", "Gentleman", role = "aut",
	            comment = "Parts of the R to C interface"),
	     person("Ross", "Ihaka", role = "aut",
	            comment = "Parts of the R to C interface"),
	     person(given = "R Core Team", role = "aut",
	            comment = "Parts of the R to C interface"),
	     person(given = "R Foundation", role = "aut",
	            comment = "Parts of the R to C interface"))
Description: Functions and data sets for actuarial science:
  modeling of loss distributions; risk theory and ruin theory;
  simulation of compound models, discrete mixtures and compound
  hierarchical models; credibility theory. Support for many additional
  probability distributions to model insurance loss size and
  frequency: 23 continuous heavy tailed distributions; the
  Poisson-inverse Gaussian discrete distribution; zero-truncated and
  zero-modified extensions of the standard discrete distributions.
  Support for phase-type distributions commonly used to compute ruin
  probabilities. Main reference: <doi:10.18637/jss.v025.i07>.
  Implementation of the Feller-Pareto family of distributions:
  <doi:10.18637/jss.v103.i06>.
Depends: R (>= 4.1.0)
Imports: stats, graphics, expint
LinkingTo: expint
Suggests: MASS
License: GPL (>= 2)
URL: https://gitlab.com/vigou3/actuar
BugReports: https://gitlab.com/vigou3/actuar/-/issues
Encoding: UTF-8
LazyData: yes
Classification/MSC-2010: 62P05, 91B30, 62G32
NeedsCompilation: yes
Packaged: 2026-03-02 22:54:40 UTC; vincent
Author: Vincent Goulet [cre, aut] (ORCID:
    <https://orcid.org/0000-0002-9315-5719>),
  Sébastien Auclair [ctb],
  Jérémy Déraspe [ctb],
  Christophe Dutang [aut] (ORCID:
    <https://orcid.org/0000-0001-6732-1501>),
  Walter Garcia-Fontes [ctb],
  Nicholas Langevin [ctb],
  Xavier Milhaud [ctb],
  Tommy Ouellet [ctb],
  Alexandre Parent [ctb],
  Mathieu Pigeon [aut],
  Louis-Philippe Pouliot [ctb],
  Jeffrey A. Ryan [aut] (Package API),
  Robert Gentleman [aut] (Parts of the R to C interface),
  Ross Ihaka [aut] (Parts of the R to C interface),
  R Core Team [aut] (Parts of the R to C interface),
  R Foundation [aut] (Parts of the R to C interface)
Maintainer: Vincent Goulet <vincent.goulet@act.ulaval.ca>
Repository: CRAN
Date/Publication: 2026-03-02 23:50:09 UTC
